# Mathew Carr 2020/06/15
# CC ZERO
# Program to decode Super Magic Drive format Sega Genesis/Mega Drive
# images into flat binary ROMs compatible with the Game Raccoon.

def chunks(lst, n):
   """Yield successive n-sized chunks from lst."""
   for i in range(0, len(lst), n):
      yield lst[i:i + n]
        
import argparse

if __name__ == "__main__":
   parser = argparse.ArgumentParser(description='Convert Super Magic Drive image to flat binary ROM image.')
   
   parser.add_argument('input',  type=str, help='input filename')
   parser.add_argument('output', type=str, help='output filename')

   args = parser.parse_args()
   
   inf = list(open(args.input, "rb").read())
   
   interleaved_data = inf[512:]
   
   length = len(interleaved_data)
   
   blocks = length / (16*1024)
   
   ouf = open(args.output, "wb")
   
   # Super Magic Drive images are arranged into 16kb blocks.
   # This block is split into an 8kb block of low bytes and a
   # 8kb block of high bytes.
   # So consider each block in turn, and then reassemble the
   # corresponding pairs of bytes when putting the two half-blocks
   # back together.
   
   for block in chunks(interleaved_data, 16*1024):
      for bytea,byteb in zip(*list(chunks(block, 8*1024))):
         ouf.write(bytes([byteb,bytea]))